@@ -1,5 +1,4 @@
-# Copyright (C) 2002, 2003, 2006, 2008, 2009, 2010 Free Software
-# Foundation, Inc.
+# Copyright (C) 2002-2012 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,7 +43,7 @@ Automake::ChannelDefs - channel definitions for Automake and helper functions
   verb ($MESSAGE, [%OPTIONS]);
   switch_warning ($CATEGORY);
   parse_WARNINGS ();
-  parse_warning ($OPTION, $ARGUMENT);
+  parse_warnings ($OPTION, $ARGUMENT);
   Automake::ChannelDefs::set_strictness ($STRICTNESS_NAME);
 
 =head1 DESCRIPTION
@@ -56,7 +55,7 @@ shorthand function to output on specific channels.
 
 =cut
 
-use 5.005;
+use 5.006;
 use strict;
 use Exporter;
 
@@ -88,7 +87,7 @@ Errors related to GNU Standards.
 
 =item C<error-gnu/warn>
 
-Errors related to GNU Standards that should be warnings in `foreign' mode.
+Errors related to GNU Standards that should be warnings in 'foreign' mode.
 
 =item C<error-gnits>
 
@@ -157,7 +156,7 @@ register_channel 'automake', type => 'fatal', backtrace => 1,
 
 register_channel 'extra-portability', type => 'warning', silent => 1;
 register_channel 'gnu', type => 'warning';
-register_channel 'obsolete', type => 'warning', silent => 1;
+register_channel 'obsolete', type => 'warning';
 register_channel 'override', type => 'warning', silent => 1;
 register_channel 'portability', type => 'warning', silent => 1;
 register_channel 'portability-recursive', type => 'warning', silent => 1;
@@ -168,6 +167,10 @@ register_channel 'verb', type => 'debug', silent => 1, uniq_part => UP_NONE,
   ordered => 0;
 register_channel 'note', type => 'debug', silent => 0;
 
+setup_channel_type 'warning', header => 'warning: ';
+setup_channel_type 'error', header => 'error: ';
+setup_channel_type 'fatal', header => 'error: ';
+
 =head2 FUNCTIONS
 
 =over 4
@@ -180,19 +183,20 @@ Display warning categories.
 
 sub usage ()
 {
-  print "Warning categories include:
-  `gnu'           GNU coding standards (default in gnu and gnits modes)
-  `obsolete'      obsolete features or constructions
-  `override'      user redefinitions of Automake rules or variables
-  `portability'   portability issues (default in gnu and gnits modes)
-  `extra-portability'  extra portability issues related to obscure tools
-  `syntax'        dubious syntactic constructs (default)
-  `unsupported'   unsupported or incomplete features (default)
-  `all'           all the warnings
-  `no-CATEGORY'   turn off warnings in CATEGORY
-  `none'          turn off all the warnings
-  `error'         treat warnings as errors
-";
+  print <<EOF;
+Warning categories include:
+  gnu                GNU coding standards (default in gnu and gnits modes)
+  obsolete           obsolete features or constructions
+  override           user redefinitions of Automake rules or variables
+  portability        portability issues (default in gnu and gnits modes)
+  extra-portability  extra portability issues related to obscure tools
+  syntax             dubious syntactic constructs (default)
+  unsupported        unsupported or incomplete features (default)
+  all                all the warnings
+  no-CATEGORY        turn off warnings in CATEGORY
+  none               turn off all the warnings
+  error              treat warnings as errors
+EOF
 }
 
 =item C<prog_error ($MESSAGE, [%OPTIONS])>
@@ -258,10 +262,6 @@ Else handle C<all> and C<none> for completeness.
 
 =cut
 
-# HACK to have `-Wextra-portability' *not* implied by `-Wall'.
-# This will go away in automake 1.12.
-my $have_extra_portability = 0;
-
 sub switch_warning ($)
 {
   my ($cat) = @_;
@@ -276,8 +276,6 @@ sub switch_warning ($)
   if ($cat eq 'all')
     {
       setup_channel_type 'warning', silent => $has_no;
-      setup_channel 'extra-portability', silent => 1
-        unless $have_extra_portability;
     }
   elsif ($cat eq 'none')
     {
@@ -294,18 +292,37 @@ sub switch_warning ($)
   elsif (channel_type ($cat) eq 'warning')
     {
       setup_channel $cat, silent => $has_no;
-      setup_channel 'portability-recursive', silent => $has_no
-        if $cat eq 'portability';
-      if ($cat eq 'portability' && $has_no)
-        {
-          setup_channel 'extra-portability', silent => 1;
-          $have_extra_portability = 0;
-        }
-      if ($cat eq 'extra-portability' && ! $has_no)
+      #
+      # Handling of portability warnings is trickier.  For relevant tests,
+      # see 'dollarvar2', 'extra-portability' and 'extra-portability3'.
+      #
+      # -Wportability-recursive and -Wno-portability-recursive should not
+      # have any effect on other 'portability' or 'extra-portability'
+      # warnings, so there's no need to handle them separately or ad-hoc.
+      #
+      if ($cat eq 'extra-portability' && ! $has_no) # -Wextra-portability
         {
+          # -Wextra-portability must enable 'portability' and
+          # 'portability-recursive' warnings.
           setup_channel 'portability', silent => 0;
           setup_channel 'portability-recursive', silent => 0;
-          $have_extra_portability = 1;
+        }
+      if ($cat eq 'portability') # -Wportability or -Wno-portability
+        {
+          if ($has_no) # -Wno-portability
+            {
+              # -Wno-portability must disable 'extra-portability' and
+              # 'portability-recursive' warnings.
+              setup_channel 'portability-recursive', silent => 1;
+              setup_channel 'extra-portability', silent => 1;
+            }
+          else # -Wportability
+            {
+              # -Wportability must enable 'portability-recursive'
+              # warnings.  But it should have no influence over the
+              # 'extra-portability' warnings.
+              setup_channel 'portability-recursive', silent => 0;
+            }
         }
     }
   else
@@ -331,7 +348,7 @@ sub parse_WARNINGS ()
     }
 }
 
-=item C<parse_warning ($OPTION, $ARGUMENT)>
+=item C<parse_warnings ($OPTION, $ARGUMENT)>
 
 Parse the argument of C<--warning=CATEGORY> or C<-WCATEGORY>.
 
@@ -347,7 +364,7 @@ sub parse_warnings ($$)
 
   foreach my $cat (split (',', $categories))
     {
-      msg 'unsupported', "unknown warning category `$cat'"
+      msg 'unsupported', "unknown warning category '$cat'"
 	if switch_warning $cat;
     }
 }
@@ -391,12 +408,10 @@ sub set_strictness ($)
     }
   else
     {
-      prog_error "level `$name' not recognized\n";
+      prog_error "level '$name' not recognized";
     }
 }
 
-1;
-
 =back
 
 =head1 SEE ALSO
@@ -409,6 +424,8 @@ Written by Alexandre Duret-Lutz E<lt>F<adl@gnu.org>E<gt>.
 
 =cut
 
+1;
+
 ### Setup "GNU" style for perl-mode and cperl-mode.
 ## Local Variables:
 ## perl-indent-level: 2
