@@ -1,5 +1,4 @@
-# Copyright (C) 1997, 2001, 2002, 2003, 2006, 2008  Free Software
-# Foundation, Inc.
+# Copyright (C) 1997-2012 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,6 +14,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Automake::Condition;
+
+use 5.006;
 use strict;
 use Carp;
 
@@ -164,11 +165,11 @@ both create the C<"FALSE"> condition).
 =cut
 
 # Keys in this hash are conditional strings. Values are the
-# associated object conditions.  This is used by `new' to reuse
+# associated object conditions.  This is used by 'new' to reuse
 # Condition objects with identical conditionals.
 use vars '%_condition_singletons';
 # Do NOT reset this hash here.  It's already empty by default,
-# and any setting would otherwise occur AFTER the `TRUE' and `FALSE'
+# and any setting would otherwise occur AFTER the 'TRUE' and 'FALSE'
 # constants definitions.
 #   %_condition_singletons = ();
 
@@ -180,6 +181,15 @@ sub new ($;@)
   };
   bless $self, $class;
 
+  for my $cond (@conds)
+    {
+      # Catch some common programming errors:
+      # - A Condition passed to new
+      confess "'$cond' is a reference, expected a string" if ref $cond;
+      # - A Condition passed as a string to new
+      confess "'$cond' does not look like a condition" if $cond =~ /::/;
+    }
+
   # Accept strings like "FOO BAR" as shorthand for ("FOO", "BAR").
   @conds = map { split (' ', $_) } @conds;
 
@@ -187,12 +197,6 @@ sub new ($;@)
     {
       next if $cond eq 'TRUE';
 
-      # Catch some common programming errors:
-      # - A Condition passed to new
-      confess "`$cond' is a reference, expected a string" if ref $cond;
-      # - A Condition passed as a string to new
-      confess "`$cond' does not look like a condition" if $cond =~ /::/;
-
       # Detect cases when @conds can be simplified to FALSE.
       if (($cond eq 'FALSE' && $#conds > 0)
 	  || ($cond =~ /^(.*)_TRUE$/ && exists $self->{'hash'}{"${1}_FALSE"})
@@ -250,7 +254,7 @@ except those of C<$minuscond>.  This is the opposite of C<merge>.
 sub strip ($$)
 {
   my ($self, $minus) = @_;
-  my @res = grep { not $minus->has ($_) } $self->conds;
+  my @res = grep { not $minus->_has ($_) } $self->conds;
   return new Automake::Condition @res;
 }
 
@@ -274,7 +278,7 @@ sub conds ($ )
 }
 
 # Undocumented, shouldn't be needed outside of this class.
-sub has ($$)
+sub _has ($$)
 {
   my ($self, $cond) = @_;
   return exists $self->{'hash'}{$cond};
@@ -289,7 +293,7 @@ Return 1 iff this condition is always false.
 sub false ($ )
 {
   my ($self) = @_;
-  return $self->has ('FALSE');
+  return $self->_has ('FALSE');
 }
 
 =item C<$cond-E<gt>true>
@@ -426,7 +430,7 @@ sub true_when ($$)
   # exists in $WHEN.
   foreach my $cond ($self->conds)
     {
-      return 0 unless $when->has ($cond);
+      return 0 unless $when->_has ($cond);
     }
   return 1;
 }
@@ -518,6 +522,8 @@ sub multiply ($@)
   return (values %res);
 }
 
+=back
+
 =head2 Other helper functions
 
 =over 4
@@ -615,6 +621,8 @@ sub conditional_negate ($)
   return $cond;
 }
 
+=back
+
 =head1 SEE ALSO
 
 L<Automake::DisjConditions>.
